@@ -1,11 +1,12 @@
+from __future__ import annotations
+
 import importlib
 import os
-from typing import List
 
 import toolz
 from tornado import web
 
-from ..utils import has_keyword
+from distributed.utils import has_keyword
 
 dirname = os.path.dirname(__file__)
 
@@ -27,7 +28,7 @@ def redirect(path):
     return Redirect
 
 
-def get_handlers(server, modules: List[str], prefix="/"):
+def get_handlers(server, modules: list[str], prefix="/"):
     prefix = prefix or ""
     prefix = "/" + prefix.strip("/")
 
